import random
import time

from thrift.protocol import TBinaryProtocol
from thrift.transport import TSocket
from ODtools.hbase_client import *
from ODtools.hbase_client.ttypes import *
from ODtools.singleton_tools import Singleton


class HBaseClient(metaclass=Singleton):
    """
    HBase tools client
    """

    def __init__(self, hbase_address: str, hbase_port: int, hbase_servers: list = None):
        """
        :param hbase_address: hbase address
        :param hbase_port: hbase port
        :param hbase_servers: hbase server node list
        """
        self.address = hbase_address
        self.port = hbase_port
        self.servers = hbase_servers
        self.reconnect()

    def reconnect(self):
        """
        reconnect hbase
        :return:
        """
        if self.servers:
            h_a, h_p = random.choice(self.servers)
            self.client = self.init_client(h_a, h_p)
        else:
            self.client = self.init_client(self.address, self.port)

    def init_client(self, address: str, port: int):
        """
        init hbase client
        :param address: hbase address
        :param port: hbase port
        :return: hbase client
        """
        for i in range(5):
            try:
                self.transport = TSocket.TSocket(address, port)
                self.transport = TTransport.TBufferedTransport(self.transport)
                protocol = TBinaryProtocol.TBinaryProtocol(self.transport)
                self.transport.open()
                self.client = THBaseService.Client(protocol)
                return self.client
            except BaseException as e:
                self.close()
                if self.servers:  # 换节点重新实例化
                    address, port = random.choice(self.servers)
                    return self.init_client(address, port)
                else:
                    raise Exception
        else:
            raise Exception

    def get_result(self, hbase_row: str, hbase_table: str) -> dict:
        """
        retrieve hbase data
        :param hbase_row: rowkey
        :param hbase_table: hbase table
        :return: result
        """
        trash = 5
        for i in range(trash):
            try:
                values = {}
                get = TGet()
                get.row = hbase_row.encode()
                result = self.client.get(hbase_table.encode(), get)
                for column in result.columnValues:
                    values[column.qualifier.decode('utf-8')] = column.value.decode('utf-8')
                return values
            except Exception as e:
                self.close()
                if i != trash - 1:
                    self.reconnect()
                    continue
                else:
                    raise e

    def put_result(self, hbase_row: str, hbase_item: dict, hbase_table: str, column_name: str = "wa") -> str:
        """
        create hbase data
        :param hbase_row: rowkey
        :param hbase_item: data
        :param hbase_table: hbase table
        :param column_name: column name
        :return:
        """
        if type(column_name) == str:
            column_name = column_name.encode(encoding='utf-8')
        if type(column_name) != bytes:
            raise Exception('Parameter error! column_name must is str or bytes')

        trash = 5
        for i in range(trash):
            try:
                coulumn_values = []
                rowkey = hbase_row.encode(encoding='utf-8')
                for key in hbase_item:
                    column = key.encode(encoding='utf-8')
                    value = str(hbase_item[key]).encode(encoding='utf-8')
                    coulumn_value = TColumnValue(column_name, column, value)
                    coulumn_values.append(coulumn_value)
                tput = TPut(rowkey, coulumn_values)
                self.client.put(hbase_table.encode(encoding='utf-8'), tput)
                return 'put success'
            except Exception as e:
                self.close()
                if i != trash - 1:
                    self.reconnect()
                    continue
                else:
                    raise e

    def delete_result(self, hbase_row: str, hbase_table: str):
        """
        delete hbase data
        :param hbase_row: rowkey
        :param hbase_table: hbase table
        :return:
        """
        trash = 5
        for i in range(trash):
            try:
                tdelete = TDelete(hbase_row.encode())
                self.client.deleteSingle(hbase_table.encode(), tdelete)
            except BaseException as e:
                self.close()
                if i != trash - 1:
                    self.reconnect()
                    continue
                else:
                    raise e

    def exists(self, hbase_row: str, hbase_table: str) -> bool:
        """
        exists rowkey
        :param hbase_row: rowkey
        :param hbase_table: hbase table
        :return:
        """
        trash = 5
        for i in range(trash):
            try:
                get = TGet()
                get.row = hbase_row.encode()
                result = self.client.exists(hbase_table.encode(), get)
                return result
            except Exception as e:
                self.close()
                if i != trash - 1:
                    self.reconnect()
                    continue
                else:
                    raise e

    def ping(self):
        """
        test hbase node
        :return:
        """
        self.init_client(self.address, self.port)
        try:
            return self.transport.isOpen()
        except BaseException as e:
            return False

    def scan_result(self, hbase_table: str, start_row: str = None):
        """
        scan hbase table data
        :param hbase_table: hbase table
        :param start_row: start rowkey
        :return:
        """
        tscan = TScan(startRow=start_row.encode() if start_row else None)
        scan_id = self.client.openScanner(hbase_table.encode(), tscan)
        row_list = self.client.getScannerRows(scan_id, 1000)
        while len(row_list) == 1000:
            for r in row_list:
                dict_data = {}
                for columnValue in r.columnValues:
                    try:
                        qualifier = columnValue.qualifier.decode()
                        value = columnValue.value.decode()
                        dict_data[qualifier] = value
                    except Exception as e:
                        print(e, "eror")
                        continue
                if r.row.decode() != start_row:  # 如果数据相同
                    start_row = r.row.decode()
                    dict_data['rowkey'] = start_row
                    yield dict_data
            try:
                tscan = TScan(startRow=str(start_row).encode() if start_row else None)
                scan_id = self.client.openScanner(str(hbase_table).encode(), tscan)
                row_list = self.client.getScannerRows(scan_id, 1000)
            except Exception as e:
                self.close()
                print("scan thrift error ", e)
                self.reconnect()
                tscan = TScan(startRow=str(start_row).encode() if start_row else None)
                scan_id = self.client.openScanner(str(hbase_table).encode(), tscan)
                row_list = self.client.getScannerRows(scan_id, 1000)
        else:
            for r in row_list:
                dict_data = {}
                for columnValue in r.columnValues:
                    try:
                        qualifier = columnValue.qualifier.decode()
                        value = columnValue.value.decode()
                        dict_data[qualifier] = value
                    except Exception as e:
                        print(e, "eror")
                        continue
                if r.row.decode() != start_row:  # 如果数据相同
                    start_row = r.row.decode()
                    dict_data['rowkey'] = start_row
                    yield dict_data

    def close(self):
        """Close the underyling transport to the HBase instance.

        This method closes the underlying Thrift transport (TCP connection).
        """
        try:
            if not self.transport.isOpen():
                return
            self.transport.close()
            del self.transport
        except BaseException as e:
            return e
        finally:
            time.sleep(2)


if __name__ == '__main__':
    pass



import os
import sys

from loguru import logger


class base_logger:
    __instance = None

    def __init__(self, log_name: str = 'default', file_path: str = './', mode: str = 'DEBUG',
                 cmd_output: bool = False, backup_count: int = 3):
        import socket
        if cmd_output:
            handler_id = logger.add(sys.stderr, level=mode)
        else:
            handler_id = logger.add(sys.stderr, level="ERROR")
        host_name = socket.gethostname()
        try:
            ip = socket.gethostbyname(host_name)
        except Exception as e:
            ip = "未知获取ip错误"
        log_file = os.path.join(file_path, log_name)
        formatter = """<green>{time:YYYY-MM-DD HH:mm:ss}</green> @zkr@ {file} @zkr@ {name} @zkr@ {level} @zkr@ {module} @zkr@ {function} @zkr@ {line} @zkr@ process-id: {process} @zkr@ """ + ip + """ @zkr@ """ + host_name + """ @zkr@ {message}"""
        logger.add("{}.log".format(log_file), format=formatter, level=mode,
                   enqueue=True, retention="5 days", encoding="utf-8",
                   rotation="50MB",buffering=10)

    def __new__(cls, log_name: str = 'default', file_path: str = './', mode: str = 'DEBUG',
                cmd_output: bool = False, backup_count: int = 3, *args, **kwargs):
        if not cls.__instance:
            cls.__instance = super(base_logger, cls).__new__(cls)
        return cls.__instance

    def info(self, msg):
        return logger.info(msg)

    def debug(self, msg):
        return logger.debug(msg)

    def warning(self, msg):
        return logger.warning(msg)

    def error(self, msg):
        return logger.error(msg)



from hashlib import md5


class SimpleHash(object):
    def __init__(self, cap, seed):
        self.cap = cap
        self.seed = seed

    def hash(self, value):
        ret = 0
        for i in range(len(value)):
            ret += self.seed * ret + ord(value[i])
        return (self.cap - 1) & ret


class BloomFilter(object):
    def __init__(self, redis_client, block_num: int = 1, key: str = 'bloomfilter'):
        """
        bloom filter
        :param redis_client: redis client
        :param block_num: block num
        :param key: redis key
        """
        self.bit_size = 1 << 31  # Redis的String类型最大容量为512M，现使用256M
        self.seeds = [5, 7, 11, 13, 31]
        # self.seeds = [5, 7, 11, 13, 31, 37, 61]
        self.server = redis_client
        # self.server = redis.StrictRedis(host=host, port=port, decode_responses=True)
        self.key = key
        self.blockNum = block_num
        self.hash_func = []
        for seed in self.seeds:
            self.hash_func.append(SimpleHash(self.bit_size, seed))

    def is_contains(self, str_input: str):
        """
        bloom filter contains string or not
        :param str_input:
        :return:
        """
        if not str_input:
            return False
        m5 = md5()
        m5.update(str_input.encode())
        str_input = m5.hexdigest()
        ret = True
        name = self.key + str(int(str_input[0:2], 16) % self.blockNum)
        for f in self.hash_func:
            loc = f.hash(str_input)
            ret = ret & self.server.getbit(name, loc)
        return ret

    def insert(self, str_input: str):
        """
        insert string to bloom filter
        :param str_input:
        :return:
        """
        m5 = md5()
        m5.update(str_input.encode())
        str_input = m5.hexdigest()
        name = self.key + str(int(str_input[0:2], 16) % self.blockNum)
        for f in self.hash_func:
            loc = f.hash(str_input)
            self.server.setbit(name, loc, 1)


if __name__ == '__main__':
    pass



from kafka import KafkaConsumer, KafkaProducer
from ODtools.singleton_tools import Singleton


class KafkaConsumerClient(metaclass=Singleton):
    def __init__(self, *topic, group_id: str, client_id: str,
                 bootstrap_servers: list = None, auto_offset_reset: str = 'latest',
                 **kwargs):

        self.consumer = KafkaConsumer(*topic,
                                      bootstrap_servers=bootstrap_servers,
                                      group_id=group_id,
                                      client_id=client_id,
                                      auto_offset_reset=auto_offset_reset,
                                      **kwargs)


class KafkaProducerClient(metaclass=Singleton):
    def __init__(self, bootstrap_servers: list = None, retries: int = 5, acks: str = 'all', **kwargs):

        self.producer = KafkaProducer(acks=acks,
                                      retries=retries,
                                      bootstrap_servers=bootstrap_servers,
                                      **kwargs)
    def send_msg(self,topic:str,value:bytes,key:bytes,**kwargs:dict):
        """
        :param topic:
        :param value:
        :param key:
        :return:
        """
        if type(topic) != str:
            topic = str(topic)
        if type(value) != bytes:
            value = value.encode()
        if type(key) != bytes:
            key = key.encode()
        if key:
            self.producer.send(topic, value,key=key).add_callback(self.send_success).add_errback(
            self.send_error)
        else:
            self.producer.send(topic, value).add_callback(self.send_success).add_errback(
                self.send_error)
        self.producer.flush()  # 批量提交

    def send_msgs(self, topic: str, values: bytes, key: bytes, **kwargs: dict):
        """
        :param topic:
        :param value:
        :param key:
        :return:
        """
        for value in values:
            if type(topic) != str:
                topic = str(topic)
            if type(value) != bytes:
                value = value.encode()
            if type(key) != bytes:
                key = key.encode()
            if key:
                self.producer.send(topic, value, key=key).add_callback(self.send_success).add_errback(
                    self.send_error)
            else:
                self.producer.send(topic, value).add_callback(self.send_success).add_errback(
                    self.send_error)
        self.producer.flush()  # 批量提交

    def send_success(self, *args, **kwargs):
        print('save success')
        return

    def send_error(self, *args, **kwargs):
        print('save error')
        return

    def close_producer(self):
        try:
            self.producer.close()
        except:
            pass


if __name__ == '__main__':
    pass



from setuptools import setup, find_packages

setup(
    name="ODtools",
    version="2.0.3",
    author="zkrPython",
    author_email="178031608@qq.com",
    description="zkrTools",
    long_description="",
    license="Apache License",
    url="https://github.com/zkr-origin-data-dpt/ODtools",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        "xlrd",
        "xlwt",
        "redis==2.10.6",
        "elasticsearch==6.1.1",
        "thrift",
        "kafka-python",
        "redis-py-cluster==1.3.6",
        "pymysql",
        "loguru",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: Indexing",
        "Topic :: Utilities",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



import random

import redis
import rediscluster
from ODtools.singleton_tools import Singleton


class RedisClient(metaclass=Singleton):
    """
    redis tools client
    """

    def __init__(self, host: str, port: int, cluster: bool = False, password: str = False):
        """
        :param host: redis host
        :param port: redis port
        :param cluster: is redis cluster or not
        """
        self.host = random.choices(host) if type(host) == list else host
        self.port = random.choices(port) if type(port) == list else port
        redis_type = rediscluster.StrictRedisCluster if cluster else redis.StrictRedis
        if password:
            self.redis = redis_type(host=self.host,
                                    port=self.port,
                                    password=password,
                                    decode_responses=True,
                                    socket_connect_timeout=5)
        else:
            self.redis = redis_type(host=self.host,
                                port=self.port,
                                decode_responses=True,
                                socket_connect_timeout=5)

        self.pipe = self.redis.pipeline()


if __name__ == '__main__':
    a = RedisClient("192.168.129.212",6390,False,"xjzfw2020").redis
    # a = RedisClient("192.168.129.213",6400,False).redis
    print(a.info())
